/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2006, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.ejb3;

import java.util.concurrent.ConcurrentHashMap;
import java.util.Collection;

/**
 * JMX JBoss 4.x Kernel abstraction for an ear.
 *
 * @author <a href="mailto:bill@jboss.org">Bill Burke</a>
 * @version $Revision: 1.17 $
 */
public class JmxEARImpl implements EAR
{
   private ConcurrentHashMap<String, Ejb3Deployment> deployments = new ConcurrentHashMap<String, Ejb3Deployment>();
   private String shortName;
   private String baseName;

   public JmxEARImpl(String shortName)
   {
      this.shortName = shortName;
      int idx = shortName.lastIndexOf('.');
      baseName = shortName.substring(0, idx);
   }

   public Collection<Ejb3Deployment> getEjbDeployments()
   {
      return deployments.values();
   }

   public void register(Ejb3Deployment deployment)
   {
      deployments.put(deployment.getDeploymentUnit().getShortName(), deployment);
   }

   public void unregister(Ejb3Deployment deployment)
   {
      deployments.remove(deployment.getDeploymentUnit().getShortName());
   }

   public Ejb3Deployment findRelativeDeployment(String relativeName)
   {
      String relativeShortName = relativeName.substring(3);
      return deployments.get(relativeShortName);
   }

   public String getShortName()
   {
      return shortName;
   }

   public String getBaseName()
   {
      return baseName;
   }

}
